import configparser
from metadata_driver_aws.config_parser import parse_config
from pathlib import Path

import pytest

from metadata_driver_aws.data_plugin import Plugin


@pytest.fixture
def aws_plugin():
    config_path = Path(__file__).parent / "resources/config.ini"
    config = parse_config(config_path.as_posix(), "metadata-driver")
    return Plugin(config)


@pytest.fixture
def test_file_path():
    test_file_path = Path(__file__).parent / "resources/test_file.md"
    return test_file_path.as_posix()



import configparser
import os


def load_config_section(file_path, section):
    try:
        assert file_path and os.path.exists(file_path), 'config file_path is required.'
        # raise
        config = parse_config(file_path, section)
        return config
    except Exception as err:
        # TODO: replace Exception with custom error class
        raise Exception("You should provide a valid config: %s" % str(err))


def parse_config(file_path, section):
    """Loads the configuration file given as parameter"""
    config_parser = configparser.ConfigParser()
    config_parser.read(file_path)
    config_dict = {}
    options = config_parser.options(section)
    for option in options:
        try:
            config_dict[option] = config_parser.get(section, option)
            if config_dict[option] == -1:
                print("skip: %s" % option)
        except Exception as err:
            print("error on %s!" % option)
            print("exception on %s" % str(err))
            config_dict[option] = None
    return config_dict



import logging

import boto3
import botocore
from metadata_driver_interface.data_plugin import AbstractPlugin
from metadata_driver_interface.exceptions import DriverError

from metadata_driver_aws.log import setup_logging
from metadata_driver_aws.s3_client import get_s3_instance


setup_logging()


class Plugin(AbstractPlugin):
    def __init__(self, config=None):
        """Initialize a :class:`~.S3_Plugin`.

        The S3_plugin is a wrapper around the boto3 S3 client and resource API.

        Configuration of the AWS credentials is handled by boto3 according to the user's environment.

        Args:
             config(dict): Configuration options
        """
        # The S3 client object
        s3_instance = get_s3_instance(config)
        self.driver = s3_instance.s3_resource
        self.sign_client = s3_instance.s3_client
        self.logger = logging.getLogger(__name__)

    @property
    def type(self):
        """str: the type of this plugin (``'AWS'``)"""
        return "AWS"

    @staticmethod
    def validate_s3_path(path):
        """Validate a path if it represents correctly a S3 path
        Args:
             path(str): The path to check.
        Raises:
            :exc:`~..DriverError`: if the file is not uploaded correctly."""
        return path.startswith("s3://")

    def parse_s3_path(self, path):
        """Validate a path if it represents correctly a S3 path
        Args:
             path(str): The path to check.
        Raises:
            :exc:`~..DriverError`: if the file is not uploaded correctly."""
        if self.validate_s3_path(path):
            bucket = path[5:].split("/", 1)[0]
            try:
                path = path[5:].split("/", 1)[1]
            except IndexError:
                path = ""
            return bucket, path
        else:
            self.logger.error(
                f"Path {path} must be a s3 url (format s3://my_bucket/my_file)"
            )
            raise DriverError

    @staticmethod
    def validate_local_path(path):
        """Validate a path if it represents correctly a local path
        Args:
             path(str): The path to check.
        Raises:
            :exc:`~..DriverError`: if the file is not uploaded correctly."""
        return not path.startswith("s3://")

    def upload(self, local_file, remote_file):
        """Upload file to a remote resource manager
        Args:
            local_file(str): The path of the file to upload.
            remote_file(str): The path of the resource manager where the file is going to be allocated.
        Raises:
            :exc:`~..DriverError`: if the file is not uploaded correctly.

        """
        self.copy(local_file, remote_file)
        self.logger.debug("Uploaded {} to {}".format(local_file, remote_file))

    def download(self, remote_file, local_file):
        """Download file from a remote resource manager
        Args:
             remote_file(str): The path in the resource manager of the file to download from.
             local_file(str): The path to the file to download to..
        Raises:
             :exc:`~..DriverError`: if the file is not downloaded correctly.
        """
        self.copy(remote_file, local_file)
        self.logger.debug("Downloaded {} to {}".format(remote_file, local_file))

    def list(self, remote_folder):
        """List all the files of a cloud directory.
        Args:
             remote_folder(str): Name of the directory to list.
        Returns:
            dict: List with the name of the file of a directory.
        Raises:
             :exc:`~..DriverError`: if the directory does not exist.
        """
        self.logger.debug("Retrieving items in {}".format(remote_folder))

        bucket, path = self.parse_s3_path(remote_folder)
        try:
            bucket_s3 = self.driver.Bucket(bucket)
            objects = bucket_s3.objects.filter(Prefix=path)
            result = [o.key for o in objects]
            return result
        except Exception as e:
            raise DriverError

    def list_buckets(self):
        bucket_iterator = self.driver.buckets.all()

        # extract only the name of the bucket
        bucket_names = [b.name for b in bucket_iterator]
        logging.debug(f"Found {bucket_names} buckets")

        return bucket_names

    def copy(self, source_path: str, dest_path: str):
        """Copy file from a path to another path.
        Args:
            source_path(str): The path of the file to be copied.
            dest_path(str): The destination path where the file is going to be allocated.
        Raises:
            :exc:`~..DriverError`: if the file is not uploaded correctly.
        """
        if not (source_path.startswith("s3://") or dest_path.startswith("s3://")):
            self.logger.error(
                "Source or destination must be a s3 url (format s3://my_bucket/my_file)"
            )
            raise DriverError
        if source_path.startswith("s3://") and dest_path.startswith("s3://"):
            self.logger.error("Source or destination must be a local directory")
            raise DriverError

        # Check if resources exists and can read
        if source_path.startswith("s3://"):
            bucket = source_path[5:].split("/", 1)[0]
            path = source_path[5:].split("/", 1)[1]
            try:
                bucket_s3 = self.driver.Bucket(bucket)
                bucket_s3.download_file(path, dest_path)
            except botocore.exceptions.ClientError as e:
                if e.response["Error"]["Code"] == "404":
                    self.logger.error(
                        f"Source file {path} in bucket {bucket} not found"
                    )
                    raise DriverError

        elif dest_path.startswith("s3://"):
            bucket = dest_path[5:].split("/", 1)[0]
            path = dest_path[5:].split("/", 1)[1]
            try:
                bucket_s3 = self.driver.Bucket(bucket)
                bucket_s3.upload_file(source_path, path)
            except botocore.exceptions.ClientError:
                self.logger.error(
                    f"There were a problem uploading local file {source_path}. Please check file exists and bucket "
                    f"{bucket} is accesible"
                )

    def generate_url(self, remote_file):
        """Generate a signed url that give access for a period of time to the resource
        Args:
            remote_file(str): The path in the resource manager of the file to give access.
        Raises:
             :exc:`~..DriverError`: if the file does not exist or if the action could not be done.
        """
        bucket, path = self.parse_s3_path(remote_file)
        url = self.sign_client.generate_presigned_url(
            ClientMethod="get_object",
            Params={"Bucket": bucket, "Key": path},
            ExpiresIn=3600 * 24,  # 1day
        )
        return url

    def delete(self, remote_file):
        """Delete a file of a remote resource manager
        Args:
             remote_file(str): The path in the resource manager of the file to delete..
        Raises:
             :exc:`~..DriverError`: if the path does not exist or if the action could not be done.
        """

        bucket, path = self.parse_s3_path(remote_file)
        delete = {"Objects": [{"Key": path}]}
        try:
            bucket_s3 = self.driver.Bucket(bucket)
            bucket_s3.delete_objects(Delete=delete)
        except Exception as e:
            raise DriverError
        self.logger.debug("Deleted {} from {}".format(path, bucket))

    def get_bucket(self, bucketname):
        # TODO: add
        pass

    def create_bucket(self, bucket):
        """Create a bucket in S3
        Args:
            bucket(str): The name of the bucket
        Raises:
             :exc:`~..DriverError`
        """
        try:
            bucket = self.driver.create_bucket(Bucket=bucket)
        except Exception:
            logging.error("Error creating bucket %s", bucket)
            raise DriverError(f"Error creating bucket {bucket}")
        self.logger.debug("Created bucket {}".format(bucket))

        return bucket

    def delete_bucket(self, bucket_name):
        """Delete a bucket in S3
        Args:
            bucket_name(str): The name of the bucket
        Raises:
             :exc:`~..DriverError`
        """
        try:
            bucket = self.driver.Bucket(bucket_name)
            for key in bucket.objects.all():
                key.delete()
            bucket.delete()
        except Exception:
            logging.error(f"Error deleting bucket {bucket_name}")
            raise DriverError
        self.logger.debug("Deleted bucket {}".format(bucket_name))

    def create_directory(self, remote_folder):
        """Create a directory in S3
        Args:
            remote_folder(str): The path of the remote directory
        Raises:
             :exc:`~..DriverError`: if the directory already exists.
        """
        bucket, path = self.parse_s3_path(remote_folder)
        if bucket == "" or path == "":
            self.logger.error("Remote folder can not be empty")
            raise DriverError
        path = path + "/" if not path.endswith("/") else path
        try:
            bucket = self.create_bucket(bucket)
            bucket.put_object(Body="", Key=path)
        except Exception:
            raise DriverError

    def retrieve_availability_proof(self):
        """TBD"""
        pass



import time
from filecmp import cmp

from metadata_driver_aws.data_plugin import Plugin


def test_complete(aws_plugin, test_file_path):
    # Create folder, upload file, list files, download file, delete file

    # Create bucket
    bucket_name = f"metadata-driver-aws-data-plugin-{int(time.time())}"
    print(f"Test bucket: {bucket_name}")
    aws_plugin.create_directory(f"s3://{bucket_name}/test")

    # List buckets
    buckets = aws_plugin.list_buckets()
    print("Buckets")
    for i, b in enumerate(buckets):
        print("\t", i, b)

    # Upload a file
    aws_plugin.upload(test_file_path, f"s3://{bucket_name}/test/TEST.md")
    files = aws_plugin.list(f"s3://{bucket_name}/test/")
    assert len(files) == 1
    assert files[0] == "test/TEST.md"

    # Get presigned_url
    sign_url = aws_plugin.generate_url(f"s3://{bucket_name}/test/TEST.md")
    assert sign_url.startswith(f"http://localhost:9000/{bucket_name}/test/TEST.md")

    # Download a file
    aws_plugin.download(
        f"s3://{bucket_name}/test/TEST.md", "/tmp/test_driver_aws_data_plugin"
    )
    assert cmp(test_file_path, "/tmp/test_driver_aws_data_plugin")

    # Delete the file
    aws_plugin.delete(f"s3://{bucket_name}/test/TEST.md")
    files = aws_plugin.list(f"s3://{bucket_name}/test/")
    assert len(files) == 0

    # Delete the bucket
    aws_plugin.delete_bucket(bucket_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("CHANGELOG.md") as history_file:
    history = history_file.read()

# Installed by pip install metadata-driver-aws
# or pip install -e .
install_requirements = [
    "coloredlogs>=15.0.1",
    "boto3>=1.18.15",
    "PyYAML>=5.4.1",
    "nevermined-metadata-driver-interface==0.2.0",
]

# Required to run setup.py:
setup_requirements = [
    "pytest-runner",
]

test_requirements = ["pytest>=6.2.4"]

# Possibly required by developers of metadata-driver-aws:
dev_requirements = [
    "bumpversion",
    "pkginfo",
    "twine",
    "watchdog",
]

setup(
    author="nevermined-io",
    author_email="root@nevermined.io",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    description="💧 Metadata S3 Driver Implementation",
    extras_require={
        "test": test_requirements,
        "dev": dev_requirements + test_requirements,
    },
    install_requires=install_requirements,
    license="Apache Software License 2.0",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="nevermined-metadata-driver-aws",
    name="nevermined-metadata-driver-aws",
    packages=find_packages(include=["metadata_driver_aws"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/nevermined-io/metadata-driver-aws",
    version="0.2.1",
    zip_safe=False,
)



import boto3
from metadatadb_driver_interface.utils import get_value
from metadata_driver_interface.exceptions import DriverError


_S3_INSTANCE = None


def get_s3_instance(config=None):
    global _S3_INSTANCE
    if _S3_INSTANCE is None:
        _S3_INSTANCE = S3Instance(config)

    return _S3_INSTANCE


class S3Instance:
    def __init__(self, config=None):
        aws_access_key = get_value("aws.access_key", "AWS_ACCESS_KEY_ID", None, config)
        aws_secret_access_key = get_value(
            "aws.secret_access_key", "AWS_SECRET_ACCESS_KEY", None, config
        )
        default_region = get_value(
            "aws.default_region", "AWS_DEFAULT_REGION", None, config
        )
        endpoint_url = get_value("aws.endpoint_url", "AWS_ENDPOINT_URL", None, config)

        if aws_access_key is None:
            raise DriverError(
                "Config value `access_key` or env variable `AWS_ACCESS_KEY_ID` not set"
            )
        if aws_secret_access_key is None:
            raise DriverError(
                "Config value `secret_access_key` or env variable `AWS_SECRET_ACCESS_KEY` not set"
            )
        if default_region is None:
            raise DriverError(
                "Config value `default_region` or env variable `AWS_DEFAULT_REGION` not set"
            )
        if endpoint_url is None:
            raise DriverError(
                "Config value `endpoint_url` or env variable `AWS_ENDPOINT_URL` not set"
            )

        try:
            self.s3_resource = boto3.resource(
                "s3",
                endpoint_url=endpoint_url,
                aws_access_key_id=aws_access_key,
                aws_secret_access_key=aws_secret_access_key,
                region_name=default_region,
            )
            self.s3_client = boto3.client(
                "s3",
                endpoint_url=endpoint_url,
                aws_access_key_id=aws_access_key,
                aws_secret_access_key=aws_secret_access_key,
                region_name=default_region,
            )
        except Exception as e:
            raise DriverError(f"Error creating and s3 instance: {str(e)}")
